@@ -3,8 +3,9 @@
 ;; Copyright (c) 2013, 2016-2017  Free Software Foundation, Inc.
 
 ;; Author: Andrew Hyatt <ahyatt@gmail.com>
+;; Homepage: https://github.com/ahyatt/emacs-websocket
 ;; Keywords: Communication, Websocket, Server
-;; Version: 1.12
+;; Version: 1.13
 ;; Package-Requires: ((cl-lib "0.5"))
 ;;
 ;; This program is free software; you can redistribute it and/or
@@ -556,7 +557,6 @@ the `websocket-error' condition."
   (websocket-debug websocket "Sending frame, opcode: %s payload: %s"
                    (websocket-frame-opcode frame)
                    (websocket-frame-payload frame))
-  (websocket-ensure-connected websocket)
   (unless (websocket-openp websocket)
     (signal 'websocket-closed (list frame)))
   (process-send-string (websocket-conn websocket)
@@ -580,21 +580,6 @@ the `websocket-error' condition."
     (setf (websocket-ready-state websocket) 'closed))
   (delete-process (websocket-conn websocket)))
 
-(defun websocket-ensure-connected (websocket)
-  "If the WEBSOCKET connection is closed, open it."
-  (unless (and (websocket-conn websocket)
-               (cl-ecase (process-status (websocket-conn websocket))
-                 ((run open listen) t)
-                 ((stop exit signal closed connect failed nil) nil)))
-    (websocket-close websocket)
-    (websocket-open (websocket-url websocket)
-                    :protocols (websocket-protocols websocket)
-                    :extensions (websocket-extensions websocket)
-                    :on-open (websocket-on-open websocket)
-                    :on-message (websocket-on-message websocket)
-                    :on-close (websocket-on-close websocket)
-                    :on-error (websocket-on-error websocket))))
-
 ;;;;;;;;;;;;;;;;;;;;;;
 ;; Websocket client ;;
 ;;;;;;;;;;;;;;;;;;;;;;
@@ -722,7 +707,7 @@ to the websocket protocol.
      conn
      (websocket-sentinel url conn key protocols extensions custom-header-alist nowait))
     (set-process-query-on-exit-flag conn nil)
-    (websocket-ensure-handshake url conn key protocols extensions custom-header-alist)
+    (websocket-ensure-handshake url conn key protocols extensions custom-header-alist nowait)
     websocket))
 
 (defun websocket-sentinel (url conn key protocols extensions custom-header-alist nowait)
@@ -731,26 +716,26 @@ to the websocket protocol.
         (websocket-debug websocket "State change to %s" change)
         (let ((status (process-status process)))
           (when (and nowait (eq status 'open))
-            (websocket-ensure-handshake url conn key protocols extensions custom-header-alist))
+            (websocket-ensure-handshake url conn key protocols extensions custom-header-alist nowait))
 
           (when (and (member status '(closed failed exit signal))
                      (not (eq 'closed (websocket-ready-state websocket))))
             (websocket-try-callback 'websocket-on-close 'on-close websocket))))))
 
-(defun websocket-ensure-handshake (url conn key protocols extensions custom-header-alist)
+(defun websocket-ensure-handshake (url conn key protocols extensions custom-header-alist nowait)
   (let ((url-struct (url-generic-parse-url url))
         (websocket (process-get conn :websocket)))
     (when (and (eq 'connecting (websocket-ready-state websocket))
-               (eq 'open (process-status conn)))
-      (process-send-string conn
-                           (format "GET %s HTTP/1.1\r\n"
-                                   (let ((path (url-filename url-struct)))
-                                     (if (> (length path) 0) path "/"))))
+               (memq (process-status conn)
+                     (list 'run (if nowait 'connect 'open))))
       (websocket-debug websocket "Sending handshake, key: %s, acceptance: %s"
                        key (websocket-accept-string websocket))
       (process-send-string conn
-                           (websocket-create-headers
-                            url key protocols extensions custom-header-alist)))))
+                           (format "GET %s HTTP/1.1\r\n%s"
+                                   (let ((path (url-filename url-struct)))
+                                     (if (> (length path) 0) path "/"))
+                                   (websocket-create-headers
+                                    url key protocols extensions custom-header-alist))))))
 
 (defun websocket-process-headers (url headers)
   "On opening URL, process the HEADERS sent from the server."
